<?php

namespace HM\BackUpWordPress;

/**
 * Class Dropbox_Service
 */
class DropboxBackUpService extends Service {

	/**
	 * Human Friendly service name
	 *
	 * @var string
	 */
	public $name = 'Dropbox';

	/**
	 * @var Dropbox The API object.
	 */
	public $dropbox;

	/**
	 * @var \Dropbox\Storage object.
	 */
	public $storage;

	/**
	 * @var \Dropbox\OAuth object.
	 */
	public $OAuth;


	/**
	 * @var Backup
	 */
	public $backup;

	/**
	 * Upload the backup to Dropbox on the hmbkp_backup_complete
	 *
	 * @see  HM_Backup::do_action
	 *
	 * @param  string $action The action received from the backup
	 *
	 * @return void
	 */
	public function action( $action, Backup $backup ) {

		$this->backup = $backup;

		if ( ( 'hmbkp_backup_complete' === $action ) && $this->get_field_value( 'DX' ) ) {

			$this->schedule->status->set_status( __( 'Uploading to Dropbox', 'backupwordpress' ) );

			$file = $backup->get_backup_filepath();

			$destination = pathinfo( $file, PATHINFO_BASENAME );

			try {

				$this->bootstrap();

				$settings = $this->fetch_settings();

				if ( isset( $settings['account_info']['quota_info'] ) ) {

					$quota_info = $settings['account_info']['quota_info'];

					// Remaining space = quota minus used space.
					$remaining = $quota_info->quota - ( $quota_info->normal + $quota_info->shared );

					// If backup is larger than remaining space or there is no space, abort.
					if ( ( filesize( $file ) > $remaining ) || ( 0 >= $remaining ) ) {

						$backup->warning( 'DX', __( 'No available space on your Dropbox account.', 'backupwordpress' ) );

						return;
					}
				}

				$folder = $this->get_field_value( 'dropbox_folder' );

				if ( empty( $folder ) ) {
					$folder = '';
				}

				if ( filesize( $file ) < 150000 ) { // 150Mb is the largest allowed for putFile

					// Upload the file with an alternative filename
					$put = $this->dropbox->putFile( $file, $destination, $folder, true );

				} else {

					$chunked = $this->dropbox->chunkedUpload( $file, $destination, $folder, true, 0, null );

				}

				$this->delete_old_backups();

			} catch ( \Dropbox\Exception $e ) {

				$backup->error( 'DX', sprintf( __( 'Error: %s', 'backupwordpress' ), $e->getMessage() ) );

			}

		}

	}

	/**
	 * Frees up space in the linked account Dropbox folder by only keeping the number of
	 * backup files defined in the max backup settings and deleting older ones.
	 */
	protected function delete_old_backups() {

		$max_backups = absint( $this->get_field_value( 'dx_max_backups' ) );

		$meta_data = $this->dropbox->metaData( $this->get_field_value( 'dropbox_folder' ) );

		$remote_files = wp_list_pluck( $meta_data['body']->contents, 'path' );

		$backup_files = array_filter( $remote_files, array( $this, 'filter_files' ) );

		if ( count( $backup_files ) <= $max_backups ) {
			return;
		}

		krsort( $backup_files );

		$files_to_delete = array_slice( $backup_files, $max_backups );

		// delete files
		$this->schedule->status->set_status( __( 'Deleting old backups from Dropbox', 'backupwordpress' ) );

		foreach ( $files_to_delete as $file ) {
			$this->dropbox->delete( $file );
		}

	}

	/**
	 * Callback to filter an array based on the backup prefix
	 *
	 * @param $element
	 *
	 * @return bool
	 */
	protected function filter_files( $element ) {

		$pattern = implode( '-', array(
				sanitize_title( str_ireplace( array( 'http://', 'https://', 'www' ), '', home_url() ) ),
				$this->schedule->get_id(),
				$this->schedule->get_type(),
			)
		);

		// if pattern is at beginning of string, returns 0, so use === to check
		return ( false === ( $pos = strpos( $element, $pattern ) ) ) ? false : true;

	}

	/**
	 * Output the Dropbox form fields.
	 * Note: Escaping of field values is done in the underlying class.
	 */
	public function form() { ?>

		<table class="form-table">

			<tbody>

			<tr>

				<th scope="row">

					<label for="<?php echo $this->get_field_name( 'DX' ); ?>"><?php _e( 'Send a copy of each backup to Dropbox', 'backupwordpress' ); ?></label>

				</th>

				<td>

					<input type="checkbox" <?php checked( $this->get_field_value( 'DX' ) ) ?> id="<?php echo $this->get_field_name( 'DX' ); ?>" name="<?php echo $this->get_field_name( 'DX' ); ?>" value="1"/>

				</td>

			</tr>

			<?php

			$schedule_id = $this->schedule->get_id();

			$settings = $this->fetch_settings();

			$dx_email = isset( $settings['account_info']['email'] ) ? $settings['account_info']['email'] : '';

			$quota_info = isset( $settings['account_info']['quota_info'] ) ? $settings['account_info']['quota_info'] : null;

			?>

			<tr>

				<th scope="row">

					<label for="dx-dropbox-unauth"><?php _e( 'Dropbox Account', 'backupwordpress' ); ?></label>

				</th>

				<td>

					<?php if ( ! $dx_email ) : ?>

						<a id="dx-dropbox-auth" href="<?php echo esc_url( wp_nonce_url( add_query_arg( array( 'action' => 'hmbkpp_dropbox_auth_step_1', 'hmbkp_schedule_id' => $schedule_id ), admin_url( 'admin-post.php' ) ), 'dropbox-auth-step1', 'dropbox-auth-step1-nonce' ) ); ?>" class="button-secondary button-right"><?php _e( 'Authenticate', 'backupwordpress' ); ?></a>

					<?php else : ?>

						<a id="dx-dropbox-unauth" href="<?php echo esc_url( wp_nonce_url( add_query_arg( array( 'action' => 'hmbkpp_dropbox_clear_auth', 'hmbkp_schedule_id' => $schedule_id ), admin_url( 'admin-post.php' ) ), 'dropbox-clear-auth', 'dropbox-clear-auth-nonce' ) ); ?>" class="button-secondary button-right"><?php _e( 'Clear authentication', 'backupwordpress' ); ?></a>

					<?php endif; ?>

				</td>

				<?php if ( $dx_email ) : ?>

					<p class="description"><?php printf( __( 'Linked account: %s', 'hmbkp' ), esc_html( $dx_email ) ); ?></p>

				<?php endif; ?>

				<?php if ( ! empty( $quota_info ) ) : ?>

					<p class="description"><?php printf( __( 'Dropbox quota info: %1$s of %2$s used', 'hmbkp' ), esc_html( size_format( $quota_info->shared + $quota_info->normal, 2 ) ), esc_html( size_format( $quota_info->quota ) ) ); ?></p>

				<?php endif; ?>

			</tr>

			<tr>

				<th scope="row">

					<label for="<?php echo $this->get_field_name( 'dropbox_folder' ); ?>"><?php _e( 'Folder', 'backupwordpress' ); ?></label>

				</th>

				<td>

					<input type="text" id="<?php echo $this->get_field_name( 'dropbox_folder' ); ?>" name="<?php echo $this->get_field_name( 'dropbox_folder' ); ?>" value="<?php echo ( $folder = $this->get_field_value( 'dropbox_folder' ) ) ? $folder : sanitize_title_with_dashes( get_bloginfo( 'name' ) ); ?>" />

					<p class="description">

						<?php printf( __( 'The folder inside %s to save the backups to, it will be created automatically if it doesn\'t already exist.', 'backupwordpress' ), '<code>/Apps/BackUpWordPress Extension/</code>' ); ?>

					</p>

				</td>

			</tr>

			<tr>

				<th scope="row">

					<label for="<?php echo $this->get_field_name( 'dx_max_backups' ); ?>"><?php _e( 'Max backups', 'backupwordpress' ); ?></label>

				</th>

				<td>

					<input  class="small-text" type="number" min="1" step="1" id="<?php echo $this->get_field_name( 'dx_max_backups' ); ?>" name="<?php echo $this->get_field_name( 'dx_max_backups' ); ?>" value="<?php echo ! $this->get_field_value( 'dx_max_backups' ) ? 3 : $this->get_field_value( 'dx_max_backups' ); ?>"/>

					<p class="description">

						<?php _e( 'The maximum number of backups to store on Dropbox.', 'backupwordpress' ); ?>

					</p>

				</td>

			</tr>

			</tbody>

		</table>

	<?php
	}

	/**
	 * Output the Dropbox Constant help text
	 */
	public static function constant() {}

	/**
	 * Define as an empty function as we are using form
	 */
	public function field() {}

	/**
	 * Validate the data before saving, if there are errors, return them to the user
	 *
	 * @param  array $new_data the new data thats being saved
	 * @param  array $old_data the old data thats being overwritten
	 *
	 * @return array           any errors encountered
	 */
	public function update( &$new_data, $old_data ) {

		$errors = array();

		if ( ! isset( $new_data['DX'] ) ) {
			return;
		} // destination was disabled

		if ( isset( $new_data['dropbox_folder'] ) ) {

			if ( empty( $new_data['dropbox_folder'] ) ) {
				$new_data['dropbox_folder'] = sanitize_title_with_dashes( get_bloginfo( 'name' ) );
			}
		}

		if ( isset( $new_data['dx_max_backups'] ) ) {

			if ( empty( $new_data['dx_max_backups'] ) || ! ctype_digit( $new_data['dx_max_backups'] ) ) {
				$errors['dx_max_backups'] = __( 'Max backups must be a number', 'backupwordpress' );
			}
		}

		return $errors;

	}

	/**
	 * The words to append to the main schedule sentence
	 *
	 * @return string The words that will be appended to the main schedule sentence
	 */
	public function display() {

		if ( $this->is_service_active() ) {
			return __( 'Dropbox', 'backupwordpress' );
		}

	}

	/**
	 * Used to determine if the service is in use or not
	 *
	 * @return bool True if service is active
	 */
	public function is_service_active() {

		return (bool) $this->get_field_value( 'DX' );

	}

	/**
	 * @param int $schedule_id
	 */
	public function bootstrap( $schedule_id = 0 ) {

		// Set your consumer key, secret and callback URL
		$key    = '86sw2ckisytw73x';
		$secret = 're00ln3qf8cf635';

		// Check whether to use HTTPS and set the callback URL
		$callback = add_query_arg( array(
			'action'                   => 'hmbkpp_dropbox_auth_step_2',
			'hmbkp_schedule_id'        => $schedule_id,
			'dropbox-auth-step2-nonce' => wp_create_nonce( 'dropbox-auth-step2' )
		), admin_url( 'admin-post.php' ) );

		// Instantiate the Encrypter and storage objects
		$encrypter = new \Dropbox\OAuth\Storage\Encrypter( 'GpJaQnfYHAfwWhpjmB94G54FphH68Qva' );

		// User ID assigned by your auth system (used by persistent storage handlers)
		$userID = get_current_user_id();

		$settings = $this->fetch_settings();

		// Instantiate the database data store and connect
		$this->storage = new \Dropbox\OAuth\Storage\WPTokens( $encrypter, $userID, $settings );

		// Create the consumer and API objects
		try {

			$this->OAuth = new \Dropbox\OAuth\Consumer\Curl( $key, $secret, $this->storage, $callback );

		} catch ( \Dropbox\Exception $e ) {

			$this->backup->warning( 'DX', sprintf( __( 'Error: %s', 'backupwordpress' ), $e->getMessage() ) );

		}

		$this->dropbox = new \Dropbox\API( $this->OAuth );

		$this->refresh_account_info();
	}

	/**
	 * Refresh the user account data ( Space quota ).
	 */
	protected function refresh_account_info() {

		$settings = $this->fetch_settings();

		$account_info = $this->dropbox->accountInfo();

		$settings['account_info'] = (array) $account_info['body'];

		update_site_option( 'hmbkpp_dx_settings', $settings );

	}

	/**
	 * Dropbox specific data to send to Intercom.
	 *
	 * @return array
	 */
	public static function intercom_data() { return array(); }

	/**
	 * Dropbox specific data to show in the admin help tab.
	 */
	public static function intercom_data_html() {}

	public function fetch_settings() {

		return get_site_option( 'hmbkpp_dx_settings' );

	}

}
Services::register( __FILE__, 'HM\\BackUpWordPress\\DropboxBackUpService' );
